@@ -223,8 +223,8 @@ resample_d.add_argument(
 )
 resample_d.add_argument(
     "--dim",
-    required=True,
-    #default=None,
+    required=False,
+    default=None,
     nargs="+",
     type=np.int,
     help="target map dimensions. e.g. 100 100 100 ",
@@ -505,11 +505,35 @@ diffmap.add_argument(
     type=float,
     help="final fit resol. (A). default= 0.0 A",
 )
-diffmap.add_argument("--usecom", action="store_true",
-                     help="if used, center-of-mass is used to superimpose maps")
+diffmap.add_argument("--nocom", action="store_true",
+                     help="if used, center-of-mass is not used during overlay")
 
 diffmap.add_argument("--usehalfmaps", action="store_true",
                      help="if used, halfmaps are used to calculate difference map")
+diffmap.add_argument(
+    "--tra",
+    required=False,
+    default=[0.0, 0.0, 0.0],
+    nargs="+",
+    type=float,
+    help="list of translation vectors. default=[0.0, 0.0, 0.0]",
+)
+diffmap.add_argument(
+    "--rot",
+    required=False,
+    default=[0.0],
+    nargs="+",
+    type=float,
+    help="list of rotations in deg. default=0.0",
+)
+diffmap.add_argument(
+    "--axr",
+    required=False,
+    default=[1, 0, 0],
+    nargs="+",
+    type=int,
+    help="list of rotation axes. default=[1,0,0]",
+)
 
 
 applymask = subparsers.add_parser(
@@ -599,13 +623,6 @@ model2map.add_argument("--dim", required=True, nargs="+",
 model2map.add_argument(
     "--cel", required=True, nargs="+", type=np.float, help="cell parameters "
 )
-#model2map.add_argument(
-#    "--bfc", required=False, default=0.0, type=float, 
-#    help="replace all atomic Bs with this. Only +ve values have an effect"
-#)
-# model2map.add_argument(
-#    "--lig", action="store_true", help="use if there is ligand, but no description"
-# )
 model2map.add_argument(
     "--lgf", required=False, default=None, type=str, help="ligand description file"
 )
@@ -613,8 +630,8 @@ model2map.add_argument(
     "--org", required=False, default=None, nargs="+", type=int, help="map origin"
 )
 model2map.add_argument(
-    "--gemmi", action="store_true", 
-    help="if used, GEMMI is used instead REFMAC for structure factor calculation"
+    "--refmac", action="store_true", 
+    help="if used, REFMAC is used instead of GEMMI for structure factor calculation"
 )
 model2map.add_argument(
     "--shift_to_boxcenter", action="store_true", 
@@ -1114,7 +1131,9 @@ def diff_map(args):
 
     difference_map(maplist=args.map, masklist=args.msk,
                    diffmapres=args.res, mode=args.mod, ncy=args.ncy,
-                   fit=args.fit, usehalfmaps=args.usehalfmaps, usecom=args.usecom, fitres=args.fitres)
+                   fit=args.fit, usehalfmaps=args.usehalfmaps, 
+                   usecom=args.nocom, fitres=args.fitres, rot=args.rot,
+                   axr=args.axr, trans=args.tra, )
 
 
 def scale_map(args):
@@ -1187,19 +1206,7 @@ def mirrormap(args):
 
 def modeltomap(args):
     from emda.emda_methods import model2map, model2map_gm, write_mrc
-
-    if args.gemmi:
-        # GEMMI for sfcalc
-        modelmap = model2map_gm(
-            modelxyz=args.mdl, 
-            resol=args.res,
-            dim=args.dim, 
-            cell=args.cel, 
-            maporigin=args.org,
-            shift_to_boxcenter=args.shift_to_boxcenter,
-            )
-        write_mrc(modelmap, "modelmap_gm.mrc", args.cel, args.org)
-    else:
+    if args.refmac:
         # REFMAC sfcalc
         modelmap = model2map(
             modelxyz=args.mdl,
@@ -1212,6 +1219,18 @@ def modeltomap(args):
             shift_to_boxcenter=args.shift_to_boxcenter,
         )
         write_mrc(modelmap, "modelmap_refmac.mrc", args.cel, args.org)
+    else:
+        # GEMMI for sfcalc
+        modelmap = model2map_gm(
+            modelxyz=args.mdl, 
+            resol=args.res,
+            dim=args.dim, 
+            cell=args.cel, 
+            maporigin=args.org,
+            shift_to_boxcenter=args.shift_to_boxcenter,
+            )
+        write_mrc(modelmap, "modelmap_gm.mrc", args.cel, args.org)
+
     
 
 
@@ -1219,6 +1238,7 @@ def mask4mmap(args):
     from emda import emda_methods as em
 
     uc, arr, orig = em.read_map(args.map)
+    em.write_mrc(arr, "arr.mrc", uc, orig)
     _ = em.mask_from_map(
         uc=uc,
         arr=arr,
