@@ -53,7 +53,7 @@ def read_mtz(mtzfile):
     return uc, df
 
 
-def get_data(struct, resol=5.0, uc=None, dim=None, maporigin=None):
+def get_data(struct, resol=5.0, gemmi=False, uc=None, dim=None, maporigin=None):
     """Returns data of a map or a model into an ndarray.
 
     Reads map data into an ndarray, or if the structure input is an atomic model,
@@ -75,6 +75,9 @@ def get_data(struct, resol=5.0, uc=None, dim=None, maporigin=None):
                     Parameter for modelmap generation. If present, the calculated map
                     will be shifted according to this information. If absent, this
                     parameter is taken as [0, 0, 0].
+                gemmi: bool, optinal
+                    Parameter for modelmap calculation. If true the map from coordinates
+                    are calculated using Gemmi. Default to False.
 
         Outputs:
             uc: float, 1D array
@@ -113,13 +116,22 @@ def get_data(struct, resol=5.0, uc=None, dim=None, maporigin=None):
                 newmodel = "new1.cif"
         if maporigin is None:
             maporigin = orig
-        modelmap = model2map(
-            modelxyz=newmodel,
-            dim=[dim, dim, dim],
-            resol=resol,
-            cell=uc,
-            maporigin=maporigin,
-        )
+        if gemmi:
+            modelmap = model2map_gm(
+                modelxyz=newmodel,
+                dim=[dim, dim, dim],
+                resol=resol,
+                cell=uc,
+                maporigin=maporigin,
+            )  
+        else:          
+            modelmap = model2map(
+                modelxyz=newmodel,
+                dim=[dim, dim, dim],
+                resol=resol,
+                cell=uc,
+                maporigin=maporigin,
+            )
         arr = modelmap
     return uc, arr, orig
 
@@ -169,16 +181,16 @@ def write_mtz(uc, arr, outfile="map2mtz.mtz", resol=None):
                          outfile=outfile, resol=resol)
 
 
-def resample_data(curnt_pix, targt_pix, targt_dim, arr):
+def resample_data(curnt_pix, targt_pix, arr, targt_dim=None):
     """Resamples a 3D array.
 
     Arguments:
         Inputs:
             curnt_pix: float list, Current pixel sizes along c, b, a.
             targt_pix: float list, Target pixel sizes along c, b a.
-            targt_dim: int list, Target sampling along z, y, x.
             arr: float, 3D array of map values.
-
+            targt_dim: int list, Target sampling along z, y, x.
+            
         Outputs:
             new_arr: float, 3D array
                 Resampled 3D array.
@@ -334,10 +346,7 @@ def mtz2map(mtzname, map_size):
     """Converts an MTZ file into MRC format.
 
     This function converts data in an MTZ file into a 3D Numpy array.
-    It combines amplitudes and phases with "Fout0" and
-    "Pout0" labels to form Fourier coefficients. If the MTZ contains
-    several aplitude columns, only the one corresponding to "Fout0"
-    will be used.
+    It combines amplitudes and phases to form Fourier coefficients.
 
     Arguments:
         Inputs:
@@ -350,8 +359,8 @@ def mtz2map(mtzname, map_size):
             outarr: float
             3D Numpy array of map values.
     """
-    data2write = maptools.mtz2map(mtzname=mtzname, map_size=map_size)
-    return data2write
+    arr = maptools.mtz2map(mtzname=mtzname, map_size=map_size)
+    return arr
 
 
 def lowpass_map(uc, arr1, resol, filter="ideal", order=4):
@@ -646,7 +655,7 @@ def halfmap_fsc_ph(half1name, half2name, filename="halffsc.txt", maskname=None):
 
         idx = np.argmin((bin_fsc - 0.8) ** 2)
         resol_rand = res_arr[idx]
-        fsc_list_ph, msk_bincount = phase_randomized_fsc(
+        fsc_list_ph, msk_bincount, _ = phase_randomized_fsc(
             arr1=arr1,
             arr2=arr2,
             mask=mask,
@@ -931,7 +940,7 @@ def get_fsc(arr1, arr2, uc):
     return res_arr, bin_fsc
 
 
-def mask_from_halfmaps(uc, half1, half2, radius=9, norm=False, iter=1, thresh=0.5):
+def mask_from_halfmaps(uc, half1, half2, radius=4, iter=1, dthresh=None):
     """Generates a mask from half maps.
 
     Generates a mask from half maps based on real space local correlation.
@@ -945,15 +954,18 @@ def mask_from_halfmaps(uc, half1, half2, radius=9, norm=False, iter=1, thresh=0.
             half2: float, 3D array
                 Half map 2 data.
             radius: integer, optional
-                Radius of integrating kernel in voxels. Default is 9.
-            norm: bool, optional
-                If true, normalized maps will be used to generate correlation mask.
-                Default is False.
+                Radius of integrating kernel in voxels. Default is 4.
             iter: integer,optional
                 Number of dilation cycles. Default is 1 cycle.
-            thresh: float, optional
-                Correlation cutoff for mask generation. Program automatically
-                decides the best value, however, user can overwrite this.
+            dthresh: float, optional
+                The halfmap densities will be thresholded at this value prior
+                calculating local correlation. If the value is not given,
+                EMDA takes this values as the value at which the cumulative
+                density probability is 0.99. This is the default. However,
+                it is recommomded that this value should be supplied
+                by the user and proven to be useful for cases those have
+                micellular densities around protein.
+                
 
         Outputs:
             mask: float, 3D array
@@ -963,18 +975,17 @@ def mask_from_halfmaps(uc, half1, half2, radius=9, norm=False, iter=1, thresh=0.
 
     arr1, arr2 = half1, half2
     # normalized maps
-    if norm:
-        hf1 = np.fft.fftshift(np.fft.fftn(arr1))
-        hf2 = np.fft.fftshift(np.fft.fftn(arr2))
-        arr1, arr2 = realsp_local.hfdata_normalized(hf1=hf1, hf2=hf2, uc=uc)
-        write_mrc(arr1, "normarr1.mrc", uc)
+    #if norm:
+    #    hf1 = np.fft.fftshift(np.fft.fftn(arr1))
+    #    hf2 = np.fft.fftshift(np.fft.fftn(arr2))
+    #    arr1, arr2 = realsp_local.hfdata_normalized(hf1=hf1, hf2=hf2, uc=uc)
     obj_maskmap = maskmap_class.MaskedMaps()
     obj_maskmap.smax = radius
     obj_maskmap.arr1 = arr1
     obj_maskmap.arr2 = arr2
+    obj_maskmap.uc = uc
     obj_maskmap.iter = iter
-    obj_maskmap.prob = thresh
-    # obj_maskmap.generate_mask(arr1, arr2, smax=radius, iter=iter, threshold=thresh)
+    obj_maskmap.dthresh = dthresh
     obj_maskmap.generate_mask()
     return obj_maskmap.mask
 
@@ -1034,7 +1045,7 @@ def mask_from_map(
     return mask
 
 
-def mask_from_atomic_model(mapname, modelname, atmrad=5):
+def mask_from_atomic_model(mapname, modelname, atmrad=5, binary_mask=False):
     """Generates a mask from atomic coordinates.
 
     Generates a mask from coordinates. First, atomic positions are
@@ -1054,6 +1065,8 @@ def mask_from_atomic_model(mapname, modelname, atmrad=5):
             atmrad: float
                 Radius of the sphere to be placed on atomic positions in Angstroms.
                 Default is 5 A.
+            binary_mask: bool
+                If used, binary mask will also be output. Default to False
 
         Outputs:
             mask: float, 3D array
@@ -1062,7 +1075,10 @@ def mask_from_atomic_model(mapname, modelname, atmrad=5):
     """
     from emda.ext.maskmap_class import mask_from_coordinates
 
-    return mask_from_coordinates(mapname=mapname, modelname=modelname, atmrad=atmrad)
+    return mask_from_coordinates(mapname=mapname, 
+                                 modelname=modelname, 
+                                 atmrad=atmrad, 
+                                 binary_mask=binary_mask)
 
 
 def sphere_kernel_softedge(radius=5):
@@ -1173,7 +1189,7 @@ def overlay_maps(
         modelres=modelres,
         nocom=nocom,
     )
-
+    return emmap1, rotmat_list, trans_list
 
 def average_maps(
     maplist,
@@ -1226,6 +1242,8 @@ def average_maps(
     """
     from emda.ext.mapfit import mapaverage
 
+    raise SystemExit("Please use 'diffmap' option for now. It calculates likelihood averagemap as well.")
+
     if axr is None:
         axr = [1, 0, 0]
     if tra is None:
@@ -1242,7 +1260,7 @@ def average_maps(
         smax=res,
         fobj=fobj,
         interp=interp,
-        fit=True,
+        fit=False,
     )
 
 
@@ -1348,18 +1366,12 @@ def realsp_correlation_mapmodel(
             resol: float
                 An argument for model based map calculation using REFMAC.
                 Resolution to calculate model based map.
+                All maps are lowpass filtered to this resolution
+                before calculating local correlation.
             kernel_size: integer, optional
                 Radius of integration kernal in pixels. Default is 5.
             mask_map: string, optional
                 Mask file to apply on correlation maps.
-                To combine with usemask option.
-            usemask: bool, optional
-                Default to False.
-                If True the map is masked before calculating correlation,
-                and the calculated correlation map is masked by the binarised mask
-                before wirting out the correlation map.
-                If the mask is not supplied, an internally calculated mask from the
-                atomic model will be used.
             norm: bool, optional
                 Defalt to False.
                 If True, correlation will be carried out on normalized maps.
@@ -1371,7 +1383,7 @@ def realsp_correlation_mapmodel(
         Outputs:
             Following maps are written out:
             modelmap.mrc - model based map (only if atomic model is given).
-            rcc_mapmodel.mrc - real space local correlation map.
+            mmcc_mapmodel.mrc - real space local correlation map.
     """
     from emda.ext import realsp_local
 
@@ -1511,7 +1523,9 @@ def mapmodel_fsc(
     return res_arr, bin_fsc
 
 
-def difference_map(maplist, diffmapres=3.0, ncy=5, mode="norm", fit=False, usehalfmaps=False, usecom=False, fitres=None, masklist=None):
+def difference_map(maplist, diffmapres=3.0, ncy=100, mode="norm", fit=False, 
+        usehalfmaps=False, usecom=False, fitres=None, masklist=None, 
+        rot=None, axr=None, trans=None):
     """Calculates difference map.
 
     Arguments:
@@ -1567,8 +1581,17 @@ def difference_map(maplist, diffmapres=3.0, ncy=5, mode="norm", fit=False, useha
     """
     from emda.ext import diffmap_with_fit
 
-    diffmap_with_fit.difference_map(maplist=maplist, masklist=masklist,
-                                    diffmapres=diffmapres, mode=mode, ncy=ncy, fit=fit, usecom=usecom, fitres=fitres, usehalfmaps=usehalfmaps)
+    diffmap_with_fit.difference_map(maplist=maplist, 
+                                    masklist=masklist,
+                                    diffmapres=diffmapres, 
+                                    mode=mode, 
+                                    ncy=ncy, 
+                                    fit=fit, 
+                                    usecom=usecom, 
+                                    fitres=fitres, 
+                                    usehalfmaps=usehalfmaps,
+                                    rot=rot, axr=axr,
+                                    trans=trans)
 
 
 def applymask(mapname, maskname, outname):
@@ -1823,7 +1846,7 @@ def mirror_map(mapname):
 
 
 def model2map(
-    modelxyz, dim, resol, cell, bfac=None, maporigin=None, ligfile=None, outputpath=None,
+    modelxyz, dim, resol, cell, bfac=None, maporigin=None, ligfile=None, outputpath=None, shift_to_boxcenter=False,
 ):
     """Calculates EM map from atomic coordinates using REFMAC5
 
@@ -1836,9 +1859,12 @@ def model2map(
             row (nystart), section (nzstart) of the unit cell. Defaults to [0, 0, 0].
         ligfile (string, optional): Name of the ligand description file. Defaults to None.
         outputpath (string, optional): Path for auxilliary files. Defaults to current
-            working directory
+            working directory.
         bfac(float, optional): Parameter for refmac. Set all atomic B values to bfac
             when it is positive. Default to None.
+        shift_to_boxcenter (bool, optional): This parameter is useful if the calculated
+            map from the model needs to be placed at the center of the box. Default to
+            False. Also, the shifted model will be written to outputpath directory.
 
     Returns:
         float ndarray: calculated model-based density array
@@ -1875,6 +1901,12 @@ def model2map(
             print("!!! Please lower the requested resolution or increase the grid dimensions !!!")
             raise SystemExit()
     # replace/add cell and write model.cif
+    if shift_to_boxcenter:
+        from emda.core.modeltools import shift_to_origin,shift_model
+        doc = shift_to_origin(modelxyz)
+        doc.write_file(outputpath+"model1.cif")
+        modelxyz = outputpath+"model1.cif"
+    # run refmac using model.cif just created
     a, b, c = cell[:3]
     structure = gm.read_structure(modelxyz)
     structure.cell.set(a, b, c, 90.0, 90.0, 90.0)
@@ -1886,6 +1918,12 @@ def model2map(
                               ligfile=ligfile,
                               bfac=bfac)
     modelmap = maptools.mtz2map(outputpath+"sfcalc_from_crd.mtz", dim)
+    if shift_to_boxcenter:
+        maporigin = None # no origin shift allowed
+        modelmap = np.fft.fftshift(modelmap) #bring modelmap to boxcenter
+        # shift model to boxcenter
+        doc = shift_model(mmcif_file=outputpath+"model.cif", shift=[a/2, b/2, c/2])
+        doc.write_file(outputpath+"emda_shifted_model.cif")
     if maporigin is None:
         maporigin = [0, 0, 0]
     else:
@@ -1900,44 +1938,61 @@ def model2map(
     return modelmap
 
 
-def model2map_gm(modelxyz, resol, dim, cell=None, maporigin=None):
-    import gemmi
+def model2map_gm(modelxyz, resol, dim, cell, maporigin=None, outputpath=None, shift_to_boxcenter=False,):
+    import gemmi, shutil
     from servalcat.utils.model import calc_fc_fft
 
+    if outputpath is None:
+        outputpath = os.getcwd()
+    outputpath = os.path.join(outputpath, 'emda_gemmifiles/')
+    print('outputpath: ', outputpath)
+    # make director for files for refmac run
+    if os.path.exists(outputpath):
+        shutil.rmtree(outputpath)
+    os.mkdir(outputpath) 
+    # check for valid sampling:
+    for i in range(3):
+        if dim[i] % 2 != 0:
+            dim[i] += 1
+    # check for minimum sampling
+    min_pix_size = resol / 2  # in Angstrom
+    min_dim = np.asarray(cell[:3], dtype="float") / min_pix_size
+    min_dim = np.ceil(min_dim).astype(int)
+    for i in range(3):
+        if min_dim[i] % 2 != 0:
+            min_dim += 1
+        if min_dim[0] > dim[0]:
+            print("Requested dims: ", dim)
+            print("Minimum dims needed (for requested resolution): ", min_dim)
+            print("!!! Please lower the requested resolution or increase the grid dimensions !!!")
+            raise SystemExit()
+    if shift_to_boxcenter:
+        from emda.core.modeltools import shift_to_origin,shift_model
+        doc = shift_to_origin(modelxyz)
+        doc.write_file(outputpath+"model1.cif")
+        modelxyz = outputpath+"model1.cif"
+    a, b, c = cell[:3]
     st = gemmi.read_structure(modelxyz)
     st.spacegroup_hm = "P 1"
-    st.cell.set(cell[0], cell[1], cell[2], 90., 90., 90.)
+    st.cell.set(a, b, c, 90., 90., 90.)
+    st.make_mmcif_document().write_file(outputpath+"model.cif")
     asu_data = calc_fc_fft(st=st, 
                            d_min=resol, 
                            source='electron', 
                            mott_bethe=True)
-
-    """ st = gemmi.read_structure(modelxyz)
-    st.spacegroup_hm = "P 1"
-    st.cell.set(cell[0], cell[1], cell[2], 90., 90., 90.)
-    dc = gemmi.DensityCalculatorX()
-    dc.d_min = resol
-    dc.rate = sample_rate = 1.5  # default
-    dc.cutoff = 1.e-5  # default
-    if bfac is None:
-        # code inherited from servalcat. Thanks Keitaro.
-        grid = resol/2/sample_rate
-        b_min = min((cra.atom.b_iso for cra in st[0].all()))
-        b_need = grid**2*8*numpy.pi**2/1.1 # Refmac's way
-        b_add = b_need - b_min
-        blur = max(0, bfac) # negative blur may cause non-positive definite in case of anisotropic Bs
-        #
-    dc.blur = blur
-    dc.addends.subtract_z()
-    dc.set_grid_cell_and_spacegroup(st)
-    dc.put_model_density_on_grid(st[0])
-    grid = gemmi.transform_map_to_f_phi(dc.grid)
-    asu_data = grid.prepare_asu_data(
-        dmin=resol, mott_bethe=True, unblur=dc.blur) """
-    griddata = asu_data.get_f_phi_on_grid(
-        asu_data.get_size_for_hkl(min_size=dim))
+    griddata = asu_data.get_f_phi_on_grid(dim)
     griddata_np = (np.array(griddata, copy=False)).transpose()
     modelmap = (np.fft.ifftn(np.conjugate(griddata_np))).real
+    if np.sum(np.asarray(modelmap.shape, 'int') - np.asarray(dim, 'int')) != 0:
+        cpix = [cell[i]/shape for i, shape in enumerate(modelmap.shape)]
+        tpix = [cell[i]/shape for i, shape in enumerate(dim)]
+        modelmap = resample_data(curnt_pix=cpix, targt_pix=tpix, arr=modelmap, targt_dim=dim)
+    if shift_to_boxcenter:
+        maporigin = None # no origin shift allowed
+        modelmap = np.fft.fftshift(modelmap) #bring modelmap to boxcenter
+        # shift model to boxcenter
+        doc = shift_model(mmcif_file=outputpath+"model.cif", shift=[a/2, b/2, c/2])
+        doc.write_file(outputpath+"emda_shifted_model.cif")
     if maporigin is None:
         maporigin = [0, 0, 0]
     else:
