@@ -553,14 +553,14 @@ def avgmaps_using_fo_nofit(emmap1, Bf_arr=None, fobj=None):
     return averagemaps
 
 
-def get_extended_signal(res_arr, signal, bin_fsc, fobj=None):
+def get_extended_signal(res_arr, signal, bin_fsc, fsc_cutoff=0.3, fobj=None):
     from scipy import stats
     from emda.ext.mapfit import rdp_algo
 
     fobj.write("\nSignal extended using B-factor \n")
     bin_fsc_masked = bin_fsc[bin_fsc > 0.0]
     res_arr_masked = res_arr[bin_fsc > 0.0]
-    dist2 = np.sqrt((bin_fsc_masked - 0.3) ** 2)
+    dist2 = np.sqrt((bin_fsc_masked - fsc_cutoff) ** 2)
     uplim = np.argmin(dist2)
     # Need to determine the low and high limits
     x = 1.0 / res_arr_masked[:uplim]
