@@ -44,6 +44,7 @@ class EmmapOverlay:
         self.cbin = None
         self.com = True
         self.com1 = None
+        self.com2 = None
         self.box_centr = None
         self.fhf_lst = []
         self.nbin = None
@@ -66,7 +67,8 @@ class EmmapOverlay:
         self.com1 = com
         if self.com:
             arr = em.shift_density(arr, np.subtract(box_centr, com))
-            print("com after centering:", em.center_of_mass_density(arr))
+            self.com2 = em.center_of_mass_density(arr)
+            print("com after centering:", self.com2)
         self.arr_lst.append(arr)
         self.fhf_lst.append(fftshift(fftn(fftshift(arr))))
         self.pixsize = uc[0] / arr.shape[0]
